﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.CompilerServices;

namespace PuzzlePlayer_Namespace
{ 
    /* The binair board consist of 1 and 0
     * This means that the possible states a cell can be are: empty, one or zero
     * To specify this in an int[,] array we will use -1 for an empty space,
     * 0 for a space with a zero and 1 for a space with a one
     * The empty space is a constant defined in the abstract Board class
     */

    internal class Binary : Board
    {

        // constructor with baordSize parameter (default is set to 8 but can be changed)
        public Binary(int boardSize = 8) : base(boardSize)
        {
            // create a clear board with the specifide size
            setBoardState(GetClearBoard(boardSize));

            description = "Binary puzzle is played on any even-numbered square grid, with some cells initially containing black or white circles. The goal of the puzzle is to fill all cells such that:\r\n- More than two circles of the same color cannot be adjacent\r\n- Each row and column must contain an equal number of black and white circles\r\n- Each row and column cannot appear multiple times on the board";
            
        }

        public override void Draw (Graphics gr, Point p, Size s)
        {
            gr.FillRectangle(Brushes.Beige, p.X, p.Y, s.Width, s.Height);
            for (int i = 0; i < BoardState.GetLength(0); i++)
            {
                for(int j = 0; j < BoardState.GetLength(1); j++)
                {
                    gr.DrawRectangle(Pens.Black,
                        p.X+i*s.Width/ BoardState.GetLength(0),
                        p.Y+j*s.Height/BoardState.GetLength(1),
                        s.Width / BoardState.GetLength(0),
                        s.Height / BoardState.GetLength(1));
                    if (BoardState[i,j] == 0)
                    {
                        gr.FillEllipse(Brushes.White,
                            (int)(p.X + ((double)i + 0.125) * s.Width / BoardState.GetLength(0)),
                            (int)(p.Y + ((double)j + 0.125) * s.Height / BoardState.GetLength(1)),
                            s.Width / BoardState.GetLength(0) * 3 / 4,
                            s.Height / BoardState.GetLength(1) * 3 / 4);
                    }
                    if (BoardState[i, j] == 1)
                    {
                        gr.FillEllipse(Brushes.Black,
                            (int)(p.X + ((double)i + 0.125) * s.Width / BoardState.GetLength(0)),
                            (int)(p.Y + ((double)j + 0.125) * s.Height / BoardState.GetLength(1)),
                            s.Width / BoardState.GetLength(0) * 3 / 4,
                            s.Height / BoardState.GetLength(1) * 3 / 4);
                    }
                }
            }
        }

        // static meathode for filling a int[,] with -1
        private static int[,] GetClearBoard(int boardSize)
        {
            int[,] result = new int[boardSize, boardSize];
            // fill the board with empty spaces (-1)
            for (int i = 0; i < boardSize; i++)
            {
                for (int j = 0; j < boardSize; j++)
                    result[i, j] = emptySpace;
            }

            return result;
        }

        /* this static methode returns true if the board is valid in a few steps
        1: It checks if the size from boardToCheck is equal to the size from the boardState variable
        2: it loops though the board and checks if one of the spaces contains something other than a -1,0 or 1
        */
        public override bool IsBoardValid(int[,] boardToCheck)
        {
            // check if the size is NOT the same
            if(!(boardToCheck.GetLength(0) ==  BoardState.GetLength(0) && boardToCheck.GetLength(1) == BoardState.GetLength(1)))
                return false;
                

            // check if any of the spaces doesn't contain a -1,0 or 1
            for(int i = 0;i < boardToCheck.GetLength(0);i++)
            {
                for(int j = 0; j < boardToCheck.GetLength(1);j++)
                {
                    switch(boardToCheck[i,j])
                    {
                        case -1:
                        case 0:
                        case 1:
                            break;
                        default:
                            return false;
                    }
                }
            }

            //check if there aren't any

            return true;
        }

        // generates a random solvable board
        public override void Generate()
        {
            // start with a clear board
            int[,] result = GetClearBoard(BoardState.GetLength(0));

            // keep adding stuff until it works?
            while (!setBoardState(result))
            {
                //add random things until it works?
                throw new NotImplementedException();

            }
        }

        // checks if the move is valid
        private static bool IsValidMove(Move m, int[,] board)
        {
            int opposite;
            if (m.changeTo == 0)
                opposite = 1;
            else
                opposite = 0;

            return MiddleCheck(m.x, m.y, board, opposite) ||
                    SideCheck(m.x, m.y, board, opposite) ||
                    EvenCheck(m.x, m.y, board, opposite);
        }
        

        // gets a list with all the possible moves
        protected override List<Move> GetSolveList(int[,] boardToSolve)
        {
            List<Move> result = new List<Move>();

            for (int i = 0; i < boardToSolve.GetLength(0); i++)
                for (int j = 0; j < boardToSolve.GetLength(1); j++)
                {
                    Move ?m = CheckMove(i, j, boardToSolve);
                    if (m != null)
                        result.Add((Move)m);
                }

            return result;
        }

        private Move? CheckMove(int x, int y, int[,] boardToSolve)
        {
            bool validForZero = false;
            bool validForOne = false;

            // empty check
            if (boardToSolve[x, y] != emptySpace)
                return null;

            // loop two times for checking 0 and 1
            for (int i = 0; i <= 1; i++)
            {
                Move m = new Move(x, y, i);

                // check if it is a valid move
                if(IsValidMove(m, boardToSolve))
                    return m;
            }

            // if both 0 and 1 fail, then the move is invalid and null is returned
            return null;
        }

        // check if the space is surrounded on both sides by the same number. If it is, the checked space should be the opposite number
        private static bool MiddleCheck(int x, int y, int[,] b, int checkFor)
        {
            // first check if x-1 and x+1 aren't out of bounds
            // after that do the check if the move is valid
            if(!(x-1 < 0 || x+1 > b.GetLength(0) - 1))
                if (b[x - 1, y] == checkFor && b[x + 1, y] == checkFor)
                    return true;

            // same for y
            if (!(y - 1 < 0 || y + 1 > b.GetLength(1) - 1))
                if (b[x, y - 1] == checkFor && b[x, y + 1] == checkFor)
                    return true;

            // return false if nothing was found
            return false;
        }

        // check if the two spaces left, right, up or down of the space are the opposite number. if so return true
        private static bool SideCheck(int x, int y, int[,] b, int checkFor)
        {
            //check the two spaces left
            if (x - 2 >= 0)
                if (b[x - 2, y] == checkFor && b[x - 1, y] == checkFor)
                    return true;

            //check the two spaces right
            if (x + 2 < b.GetLength(0))
                if (b[x + 2, y] == checkFor && b[x + 1, y] == checkFor)
                    return true;

            //check the two spaces down
            if (y - 2 >= 0)
                if (b[x , y- 2] == checkFor && b[x , y- 1] == checkFor)
                    return true;

            //check the two spaces up
            if (y + 2 < b.GetLength(1))
                if (b[x , y+ 2] == checkFor && b[x , y+ 1] == checkFor)
                    return true;  

            return false;
        }

        // every row and colom should have an even number of 1'boardSize and 0'boardSize. So if the total number of 1'boardSize in a row is equal to half the with of the row a 0 should be filled in.
        private static bool EvenCheck(int x, int y, int[,] b, int checkFor)
        {
            // check for a row and colum (provided that the width and height of the board is the same)
            int countRow = 0, countCol = 0;
            for (int i = 0; i < b.GetLength(0); i++)
            {
                if(b[i,y] == checkFor) countRow++;
                if(b[x,i] == checkFor) countCol++;
            }

            // check if the total number of oppisite numbers is equal to half te widht/heigt
            if (countRow == b.GetLength(0) / 2 || countCol == b.GetLength(1) / 2)
                return true;

            return false;
        }
    }
}